"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInputData = exports.checkProviderList = exports.providerAccessFormat = exports.providerCollection = exports.providerObject = exports.providerArray = void 0;
var i18n_1 = require("../../utils/i18n");
exports.providerArray = ['alibaba', 'baidu', 'huawei', 'aws', 'azure', 'google', 'tencent'];
exports.providerObject = {
    alibaba: i18n_1.default.__('Alibaba Cloud'),
    baidu: i18n_1.default.__('Baidu Cloud'),
    huawei: i18n_1.default.__('Huawei Cloud'),
    aws: i18n_1.default.__('AWS Cloud'),
    azure: i18n_1.default.__('Azure Cloud'),
    google: i18n_1.default.__('Google Cloud'),
    tencent: i18n_1.default.__('Tencent Cloud')
};
exports.providerCollection = {
    alibaba: [
        {
            type: 'input',
            message: 'AccountID',
            name: 'AccountID',
            default: '' // 默认值
        },
        {
            type: 'input',
            message: 'AccessKeyID',
            name: 'AccessKeyID',
            default: '' // 默认值
        },
        {
            type: 'input',
            message: 'AccessKeySecret',
            name: 'AccessKeySecret',
            default: '' // 默认值
        }
    ],
    aws: [
        {
            type: 'input',
            message: 'AccessKeyID',
            name: 'AccessKeyID',
            default: '' // 默认值
        },
        {
            type: 'input',
            message: 'SecretAccessKey',
            name: 'SecretAccessKey',
            default: '' // 默认值
        }
    ],
    huawei: [
        {
            type: 'input',
            message: 'AccessKeyID',
            name: 'AccessKeyID',
            default: '' // 默认值
        },
        {
            type: 'input',
            message: 'SecretAccessKey',
            name: 'SecretAccessKey',
            default: '' // 默认值
        }
    ],
    azure: [
        {
            type: 'input',
            message: 'KeyVault',
            name: 'KeyVault',
            default: '' // 默认值
        },
        {
            type: 'input',
            message: 'Secret',
            name: 'Secret',
            default: '' // 默认值
        }
    ],
    baidu: [
        {
            type: 'input',
            message: 'AccessKeyID',
            name: 'AccessKeyID',
            default: '' // 默认值
        },
        {
            type: 'input',
            message: 'SecretAccessKey',
            name: 'SecretAccessKey',
            default: '' // 默认值
        }
    ],
    google: [
        {
            type: 'input',
            message: 'AccountID',
            name: 'AccountID',
            default: '' // 默认值
        },
        {
            type: 'input',
            message: 'PrivateKeyData',
            name: 'PrivateKeyData',
            default: '' // 默认值
        }
    ],
    tencent: [
        {
            type: 'input',
            message: 'AccountID',
            name: 'AccountID',
            default: '' // 默认值
        },
        {
            type: 'input',
            message: 'SecretID',
            name: 'SecretID',
            default: '' // 默认值
        },
        {
            type: 'input',
            message: 'SecretKey',
            name: 'SecretKey',
            default: '' // 默认值
        }
    ]
};
exports.providerAccessFormat = {
    alibaba: ['AccountID', 'AccessKeyID', 'AccessKeySecret'],
    aws: ['AccessKeyID', 'SecretAccessKey'],
    baidu: ['AccessKeyID', 'SecretAccessKey'],
    huawei: ['AccessKeyID', 'SecretAccessKey'],
    azure: ['KeyVault', 'Secret'],
    tencent: ['AccountID', 'SecretID', 'SecretKey'],
    google: ['AccountID', 'PrivateKeyData']
};
exports.checkProviderList = [
    {
        type: 'list',
        name: 'provider',
        message: i18n_1.default.__('Please select a provider:'),
        choices: [
            { name: i18n_1.default.__('Alibaba Cloud (alibaba)'), value: 'alibaba' },
            { name: i18n_1.default.__('AWS (aws)'), value: 'aws' },
            { name: i18n_1.default.__('Azure (azure)'), value: 'azure' },
            { name: i18n_1.default.__('Baidu Cloud (baidu)'), value: 'baidu' },
            { name: i18n_1.default.__('Google Cloud (google)'), value: 'google' },
            { name: i18n_1.default.__('Huawei Cloud (huawei)'), value: 'huawei' },
            { name: i18n_1.default.__('Tencent Cloud (tencent)'), value: 'tencent' }
        ]
    }
];
function getInputData(program) {
    var inputSecretCheck = {};
    if (program.AccountID) {
        inputSecretCheck['AccountID'] = program.AccountID;
    }
    if (program.AccessKeyID) {
        inputSecretCheck['AccessKeyID'] = program.AccessKeyID;
    }
    if (program.AccessKeySecret) {
        inputSecretCheck['AccessKeySecret'] = program.AccessKeySecret;
    }
    if (program.SecretID) {
        inputSecretCheck['SecretID'] = program.SecretID;
    }
    if (program.SecretKey) {
        inputSecretCheck['SecretKey'] = program.SecretKey;
    }
    if (program.SecretAccessKey) {
        inputSecretCheck['SecretAccessKey'] = program.SecretAccessKey;
    }
    return inputSecretCheck;
}
exports.getInputData = getInputData;
//# sourceMappingURL=common.js.map